/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "drawui.h"
#include "drawshape.h"
#include "assert/assert.h"
#include "game/time.h"
#include "display/draw/drawtext.h"
#include "ui/textbox.h"
#include "ui/testmenu.h"

void drawUIBox(
  const uint16_t x,
  const uint16_t y,
  const uint16_t width,
  const uint16_t height,
  const uint8_t color,
  const bool_t fill
) {
  assertTrue(x < FRAME_WIDTH, "Box is too far right.");
  assertTrue(y < FRAME_HEIGHT, "Box is too far down.");
  assertTrue(width > 2, "Box is too narrow.");
  assertTrue(height > 2, "Box is too short.");
  assertTrue(x + width <= FRAME_WIDTH, "Box is too wide.");
  assertTrue(y + height <= FRAME_HEIGHT, "Box is too tall.");

  // Draw the top and bottom borders
  for(uint16_t i = 0; i < width; i++) {
    FRAME_BUFFER[y * FRAME_WIDTH + x + i] = '-';
    FRAME_BUFFER[(y + height - 1) * FRAME_WIDTH + x + i] = '-';
    FRAME_COLOR[y * FRAME_WIDTH + x + i] = color;
    FRAME_COLOR[(y + height - 1) * FRAME_WIDTH + x + i] = color;
  }

  // Draw the left and right borders
  for(uint16_t i = 0; i < height; i++) {
    FRAME_BUFFER[(y + i) * FRAME_WIDTH + x] = '|';
    FRAME_BUFFER[(y + i) * FRAME_WIDTH + x + width - 1] = '|';
    FRAME_COLOR[(y + i) * FRAME_WIDTH + x] = color;
    FRAME_COLOR[(y + i) * FRAME_WIDTH + x + width - 1] = color;
  }

  // Draw the corners
  FRAME_BUFFER[y * FRAME_WIDTH + x] = '+';
  FRAME_BUFFER[y * FRAME_WIDTH + x + width - 1] = '+';
  FRAME_BUFFER[(y + height - 1) * FRAME_WIDTH + x] = '+';
  FRAME_BUFFER[(y + height - 1) * FRAME_WIDTH + x + width - 1] = '+';
  FRAME_COLOR[y * FRAME_WIDTH + x] = color;
  FRAME_COLOR[y * FRAME_WIDTH + x + width - 1] = color;
  FRAME_COLOR[(y + height - 1) * FRAME_WIDTH + x] = color;
  FRAME_COLOR[(y + height - 1) * FRAME_WIDTH + x + width - 1] = color;

  // Fill the inside
  if(!fill) return;
  drawBox(x + 1, y + 1, width - 2, height - 2, ' ', COLOR_BLACK);
}

void drawUIMenu(
  const menu_t *menu,
  const uint16_t x,
  const uint16_t y,
  const uint16_t width,
  const char_t* strings[],
  const int16_t stringCount,
  const uint8_t cursorColor,
  const uint8_t textColor
) {
  assertTrue(menu, "Menu cannot be NULL.");
  assertTrue(strings, "Strings cannot be NULL.");
  assertTrue(stringCount > 0, "String count must be greater than 0.");
  assertTrue(menu->rows > 0, "Rows must be greater than 0.");
  assertTrue(menu->columns > 0, "Columns must be greater than 0.");
  assertTrue(width*2 > menu->columns, "Width must be greater than columns.");

  uint16_t stringWidth = (width / menu->columns) + 1;// +1 for cursor and padding

  // Draw the menu
  for(uint16_t row = 0; row < menu->rows; row++) {
    for(uint16_t col = 0; col < menu->columns; col++) {
      uint16_t index = (row * menu->columns) + col;
      if(index >= stringCount) break;

      drawText(
        strings[index],
        -1,
        x + (col * stringWidth) + 1,
        y + row,
        textColor
      );
    }
  }

  // Draw the cursor
  size_t i = (x + (menu->x * stringWidth)) + ((y + menu->y) * FRAME_WIDTH);
  FRAME_BUFFER[i] = '>';
  FRAME_COLOR[i] = cursorColor;
}

void drawUIMenuBox(
  const menu_t *menu,
  const uint16_t x,
  const uint16_t y,
  const uint16_t width,
  const uint16_t height,
  const char_t* strings[],
  const int16_t stringCount,
  const uint8_t cursorColor,
  const uint8_t textColor,
  const uint8_t boxColor
) {
  drawUIBox(x, y, width, height, boxColor, true);

  // drawUIMenu(
  //   menu,
  //   x + 1, y + 1, width - 2,
  //   strings, stringCount,
  //   cursorColor, textColor
  // );
}

void drawUITextbox() {
  if(!textboxIsOpen()) return;

  // Border
  drawUIBox(
    0, FRAME_HEIGHT - DRAW_UI_TEXTBOX_HEIGHT,
    DRAW_UI_TEXTBOX_WIDTH, DRAW_UI_TEXTBOX_HEIGHT,
    COLOR_MAGENTA, true
  );

  // Title
  drawText(
    TEXTBOX.title,
    -1,
    2, FRAME_HEIGHT - DRAW_UI_TEXTBOX_HEIGHT,
    COLOR_WHITE
  );

  // Text
  drawText(
    TEXTBOX.text,
    TEXTBOX.textIndex,
    1, FRAME_HEIGHT - DRAW_UI_TEXTBOX_HEIGHT + 1,
    COLOR_WHITE
  );

  // Blinking cursor
  memset(&FRAME_BUFFER[DRAW_UI_TEXTBOX_CURSOR_POS], ' ', 3);
  memset(&FRAME_COLOR[DRAW_UI_TEXTBOX_CURSOR_POS], COLOR_WHITE, 3);
  if(TEXTBOX.textIndex < TEXTBOX.textLength) return;
  
  int32_t blink = (int32_t)(TIME.time * DRAW_UI_TEXTBOX_BLINKS_PER_SECOND) % 2;
  FRAME_BUFFER[DRAW_UI_TEXTBOX_CURSOR_POS + 1] = blink ? '>' : ' ';
  FRAME_BUFFER[DRAW_UI_TEXTBOX_CURSOR_POS + 2] = blink ? ' ' : '>';
}

void drawUITestMenu() {
  const char_t* strings[] = {
    "Option 1",
    "Option 2",
    "Option 3",
    "Option 4",
    "Option 5",
    "Option 6",
    "Option 7",
    "Option 8",
    "Option 9",
    "Option 10",
    "Option 11",
    "Option 12",
    "Option 13",
    "Option 14",
    "Option 15",
    "Option 16",
    "Option 17",
    "Option 18",
    "Option 19",
    "Option 20",
    "Option 21",
    "Option 22",
    "Option 23",
    "Option 24",
    "Option 25",
    "Option 26",
    "Option 27",
    "Option 28",
    "Option 29",
    "Option 30"
  };

  drawUIMenuBox(
    &TEST_MENU.menu,
    0, 0,
    FRAME_WIDTH, 5,
    strings, sizeof(strings) / sizeof(char_t*),
    COLOR_CYAN, COLOR_WHITE, COLOR_MAGENTA
  );
}